@@ -549,9 +549,9 @@ class WebDriver(webdriver.Remote):
             'optional_intent_arguments': 'optionalIntentArguments',
             'stop_app_on_reset': 'stopAppOnReset'
         }
-        for key in arguments.iterkeys():
-            if opts.has_key(key):
-                data[arguments[key]] = opts[key]
+        for key, value in arguments.items():
+            if key in opts:
+                data[value] = opts[key]
         self.execute(Command.START_ACTIVITY, data)
         return self
 
@@ -693,6 +693,23 @@ class WebDriver(webdriver.Remote):
         self.execute(Command.TOGGLE_LOCATION_SERVICES, {})
         return self
 
+    def set_location(self, latitude, longitude, altitude):
+        """Set the location of the device
+
+        :Args:
+         - latitude - String or numeric value between -90.0 and 90.00
+         - longitude - String or numeric value between -180.0 and 180.0
+         - altitude - String or numeric value
+        """
+        data = {
+            "location": {
+                "latitude": str(latitude),
+                "longitude": str(longitude),
+                "altitude": str(altitude)
+            }
+        }
+        self.execute(Command.SET_LOCATION, data)
+        return self
 
     def _addCommands(self):
         self.command_executor._commands[Command.CONTEXTS] = \
@@ -765,22 +782,14 @@ class WebDriver(webdriver.Remote):
         self.command_executor._commands[Command.GET_ACTIVE_IME_ENGINE] = \
             ('GET', '/session/$sessionId/ime/active_engine')
         self.command_executor._commands[Command.REPLACE_KEYS] = \
-            ('POST', '/session/$sessionId/appium/element/$elementId/replace_value')
+            ('POST', '/session/$sessionId/appium/element/$id/replace_value')
         self.command_executor._commands[Command.GET_SETTINGS] = \
             ('GET', '/session/$sessionId/appium/settings')
         self.command_executor._commands[Command.UPDATE_SETTINGS] = \
             ('POST', '/session/$sessionId/appium/settings')
         self.command_executor._commands[Command.TOGGLE_LOCATION_SERVICES] = \
             ('POST', '/session/$sessionId/appium/device/toggle_location_services')
-
-
-# monkeypatched method for WebElement
-def set_value(self, value):
-    """Set the value on this element in the application
-    """
-    data = {
-        'elementId': self.id,
-        'value': [value],
-    }
-    self._execute(Command.SET_IMMEDIATE_VALUE, data)
-    return self
+        self.command_executor._commands[Command.SET_LOCATION] = \
+            ('POST', '/session/$sessionId/location')
+        self.command_executor._commands[Command.LOCATION_IN_VIEW] = \
+            ('GET', '/session/$sessionId/element/$id/location_in_view')
